<?php
/* $Id: Planworld_User.php,v 1.3 2001/08/03 14:03:55 seth Exp $ */

/**
 * User class for planworld.
 * @author Seth Fitzsimmons
 */
class Planworld_User {
    
    /** general user information */
    // TODO: defaults should be set in config.php
    var $archive = 'N';  // tri-state (Y, P, N)
    var $lastLogin = 'UNIX_TIMESTAMP()';
    var $lastUpdate = 0;
    var $plan;  // Plan object
    var $planwatch;  // Planwatch object
    var $snitchDisplayNum = 25;
    var $snitchEnabled = 0;  // mixed
    var $theme = PW_DEFAULT_THEME;
    var $userID;
    var $userInfo;
    var $username;
    var $views = 0;
    var $watchOrder = 'alph';

    /** flags (boolean) */
    var $admin = false;
    var $connected = false;
    var $remoteUser = false;
    var $snitch = false;
    var $world = false;

    var $dbh;
    
    /**
     * Constructor.
     * @param user User to initialize.
     * @param new User is a new user.
     * @public
     * @returns bool
     */
    function Planworld_User ($user, $new=false) {
        if (is_string($user)) {
            $this->username = $user;
        } else if (is_int($user)) {
            $this->userID = $user;
        }

        if ($new && isset($this->username)) {
            return $this->save(true);
        } else {
            return $this->load();
        }
    }
    
    /**
     * Attempts to open a persistent connection to the SQL server.
     *
     * @returns bool
     */
    function connect() {
        if (!$this->connected) {
            include_once 'DB.php';

            $this->dbh = &DB::connect(PW_DB_TYPE . '://' . PW_DB_USER . ':' . PW_DB_PASS . '@' . PW_DB_HOST . '/' . PW_DB_NAME, true);
            if (DB::isError($this->dbh) || DB::isWarning($this->dbh)) {
                return false;
            }
 
            $this->connected = true;
        }
        return true;
    }

    /**
     * Loads user information.
     * @public
     * @returns bool
     */
    function load () {
        
        /* If we're not already connected, invoke the connect() method. */
        if (!$this->connected) {
            if (!$this->connect()) return false;
        }

        /* Build the query */
        $query = 'SELECT * FROM users WHERE ';
        if (isset($this->username)) $query .= "username='{$this->username}'";
        else if (isset($this->userID)) $query .= "id='{$this->userID}'";
        else return false;
        $query .= ' LIMIT 1';

        $result = $this->dbh->query($query);

        if (isset($result) && !DB::isError($result)) {
            $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
            if (DB::isError($row)) return false;

            $this->userID = $row['id'];
            $this->username = $row['username'];
            $this->remoteUser = ($row['remote'] == 'Y') ? true : false;
            $this->world = ($row['world'] == 'Y') ? true : false;
            $this->snitch = ($row['snitch'] == 'Y') ? true : false;
            $this->archive = $row['remote'];
            $this->snitchDisplayNum = $row['snitchViews'];
            $this->views = $row['totalViews'];
            $this->watchOrder = $row['watchOrder'];
            $this->theme = $row['themeID'];
            $this->snitchEnabled = $row['snitchOn'];
            $this->lastLogin = $row['lastOn'];
            $this->lastUpdate = $row['lastUpdate'];
        } else {
            return false;
        }

        // data has now been synchronized
        $this->changed = false;
        return true;
    }
    
    /**
     * Saves user information.
     * @param new User is to be created.
     * @public
     * @returns bool
     */
    function save ($new=false) {
        if ($this->changed) {
            /* If we're not already connected, invoke the connect() method. */
            if (!$this->connected) {
                if (!$this->connect()) return false;
            }

            $info = array();
            $info['username'] = "'{$this->username}'";
            $info['remote'] = ($this->remoteUser) ? "'Y'" : "'N'";
            $info['world'] = ($this->world) ? "'Y'" : "'N'";
            $info['snitch'] = ($this->snitch) ? "'Y'" : "'N'";
            $info['snitchViews'] = &$this->snitchDisplayNum;
            $info['archive'] = ($this->archive) ? "'Y'" : "'N'";
            $info['totalViews'] = &$this->views;
            $info['watchOrder'] = "'{$this->watchOrder}'";
            $info['themeID'] = &$this->theme;
            $info['snitchOn'] = &$this->snitchEnabled;
            $info['lastOn'] = &$this->lastLogin;
            $info['lastUpdate'] = &$this->lastUpdate;

            /* assemble the query */  // XXX untested.  may crash and burn without warning
            if ($new) {
                $query = 'INSERT INTO user (username, remote, world, snitch, snitchViews, archive, totalViews, watchOrder, themeID, snitchOn, lastOn, lastUpdate)';
                $query .= 'VALUES (';
                $query .= implode(',', $info);
                $query .= ')';
            } else {
                $pair = array();
                foreach ($info as $key => $value) {
                    $pair[] = implode('=', $info[$key]);
                }
                $query = 'UPDATE user SET ';
                $query .= implode(',', $pair);
                if (isset($this->username)) $query .= " WHERE username='{$this->username}'";
                else if (isset($this->userID)) $query .= " WHERE id='{$this->userID}'";
                else return false;
            }

            /* execute the query */
            $result = $this->dbh->query($query);
            if (DB::isError($result)) {
                return false;
            }
            
            // data has now been synchronized
            $this->changed = false;
        }
        return true;
    }
    
    /**
     * Clear this user's snitch list.
     * @private
     * @returns void
     */
    function clearSnitch () {
        /* If we're not already connected, invoke the connect() method. */
        if (!$this->connected) {
            if (!$this->connect()) return false;
        }

        $this->snitch = false;
        $this->snitchEnabled = -1;
        $this->changed = true;

        $query = "DELETE FROM snitch WHERE userId={$this->userID}";
        $this->dbh->query($query);
    }
    
    /**
     * Increment the number of plan views for this user.
     * @public
     * @returns int
     */
    function incrementViews () {
        $this->views++;
        $this->changed = true;
        return $this->views;
    }
    
    /**
     * Starts this user's snitch list.
     * @private
     * @returns void
     */
    function startSnitch () {
        /* If we're not already connected, invoke the connect() method. */
        if (!$this->connected) {
            if (!$this->connect()) return false;
        }

        $this->snitch = true;
        $this->snitchEnabled = 'UNIX_TIMESTAMP()';
        $this->changed = true;
    }
    
    /* flags */
    
    /**
     * Is this user an admin?
     * @public
     * @returns bool
     */
    function isAdmin () {
        return $this->admin;
    }
    
    /**
     * Is this user's plan archived?
     * @public
     * @returns bool
     */
    function isArchived () {
        return ($this->archive == 'Y' || $this->archive == 'P') ? true : false;
    }
    
    /**
     * Is this user's plan archived publicly?
     * @public
     * @returns bool
     */
    function isArchivedPublicly () {
        return ($this->archive == 'Y') ? true : false;
    }
    
    /**
     * Has this user been changed?
     * @public
     * @returns bool
     */
    function isChanged () {
        return $this->changed;
    }
    
    /**
     * Is this user remote?
     * @public
     * @returns bool
     */
    function isRemoteUser () {
        return $this->remoteUser;
    }
    
    /**
     * Is this user's plan world-accessible?
     * @public
     * @returns bool
     */
    function isWorld () {
        return $this->getWorld();
    }
    
    /* getX and setX */
    
    /**
     * Get archive settings.
     * @public
     * @returns string
     */
    function getArchive () {
        return $this->archive;
    }
    
    /**
     * Set this user's archival settings
     * @param val (Y) Public / (P) private / (N) off archiving
     * @public
     * @returns bool
     */
    function setArchive ($val) {
        $val = strtoupper($val);
        if ($val == 'Y' || $val == 'P' || $val == 'N') {
            $this->archive = $val;
            $this->changed = true;
            return true;
        } else {
            return false;
        }
    }
    
    /**
     * Return the last time this user logged in.
     * @public
     * @returns int
     */
    function getLastLogin () {
        return $this->lastLogin;
    }
    
    /**
     * Return the last time this user updated his/her plan.
     * @public
     * @returns int
     */
    function getLastUpdate () {
        return $this->lastUpdate;
    }
    
    /**
     * Return the text of this user's plan.
     * @param date Timestamp of the requested plan
     * @param revision Revision of the requested plan
     * @public
     * @returns Plan
     */
    function getPlan ($date='', $revision='') {
        // TODO: implement
        if (empty($date) && empty($revision)) {
            return $this->plan;
        } else {
            // fetch the plan from the archives
        }
    }
    
    /**
     * Is this user snitch registered?
     * @public
     * @returns bool
     */
    function getSnitch () {
        return $this->snitch;
    }
    
    /**
     * Set snitch registration status.
     * @param val True / false registration
     * @public
     * @returns void
     */
    function setSnitch ($val) {
        $this->changed = true;
        if (!$val && $this->snitch) {
            $this->clearSnitch();
        } else if ($val != $this->snitch) {
            $this->startSnitch();
        } else {
            $this->changed = false;
        }
        $this->snitch = $val;
    }
    
    /**
     * Get number of users to display on snitch list.
     * @public
     * @returns int
     */
    function getSnitchDisplayNum () {
        return $this->snitchDisplayNum;
    }
    
    /**
     * Set number of users to display on snitch list.
     * @param num Number of users to display
     * @public
     * @returns void
     */
    function setSnitchDisplayNum ($num) {
        $this->snitchDisplayNum = $num;
        $this->changed = true;
    }
    
    /**
     * Returns a list of users who have referenced this user's plan.
     * @public
     * @returns array
     */
    function getSnoopRefs () {
        // no object variable needs to be instantiated (unless shared) as this needs
        // to be reloaded each time and is only relevant to the user
        include_once(dirname(__FILE__) . '/Snoop.php');
        return Snoop::getRefs($this);
    }
    
    /**
     * Get this user's preferred theme.
     * @public
     * @returns int
     */
    function getTheme () {
        return $this->theme;
    }
    
    /**
     * Set the theme that this user uses.
     * @param theme Theme to use
     * @public
     * @returns void
     */
    function setTheme ($theme) {
        $this->theme = $theme;
    }

    /**
     * Returns this user's userid.
     * @public
     * @returns int
     */
    function getUserID () {
        return $this->userID;
    }
    
    /**
     * Returns this user's username.
     * @public
     * @returns string
     */
    function getUsername () {
        return $this->username;
    }
    
    /**
     * Get information on this user.
     * @public
     * @returns array
     */
    function getUserInfo () {
        // XXX load on demand and set $this->changed;
        // TODO: implement
        return $this->userInfo;
    }
    
    /**
     * Returns this user's planwatch ordering.
     * @public
     * @returns string
     */
    function getWatchOrder () {
        return $this->watchOrder;
    }
    
    /**
     * Set this user's planwatch ordering.
     * @param type Type of ordering
     * @public
     * @returns void
     */
    function setWatchOrder ($type) {
        $this->watchOrder = $type;
        $this->changed = true;
    }
    
    /**
     * Is this user's plan world-accessible?
     * @public
     * @returns bool
     */
    function getWorld () {
        return $this->world;
    }
    
    /**
     * Set this user's world-accessibility
     * @param val True / false world-accessibility
     * @public
     * @returns void
     */
    function setWorld ($val) {
        $this->world = $val;
        $this->changed = true;
    }
}
?>